package br.com.caelum.tubaina.builder.replacer;

import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import br.com.caelum.tubaina.Chunk;
import br.com.caelum.tubaina.chunk.ParagraphChunk;

public class ParagraphReplacer implements Replacer {

	private final Pattern pattern;
	
	private final Pattern tagPattern = Pattern.compile("(?s)(?i)\\A\\[(.+)\\].*\\[/\\1\\]");

	private final boolean isInsideItem;
	
	public ParagraphReplacer(String specialTerms, boolean isInsideItem) {
		this.isInsideItem = isInsideItem;
		this.pattern = Pattern.compile("(?s)(?i)\\A\\s*(.+?)(\\n(\\s)*?\\n|\\s*\\[(" +
				specialTerms +
				")|\\s*\\z)");
	}
	public ParagraphReplacer(String specialTerms) {
		this(specialTerms, false);
	}

	@Override
	public String execute(String text, List<Chunk> chunks) {
		Matcher matcher = pattern.matcher(text);
		matcher.find();
		chunks.add(createChunk(matcher));
		return text.substring(matcher.end(1));
	}

	@Override
	public boolean accepts(String text) {
		Matcher tagMatcher = tagPattern.matcher(text);
		if (tagMatcher.find())
			return false;
		Matcher matcher = pattern.matcher(text);
		if (matcher.find()) {
			if (matcher.group().trim().length() > 0) {
				return true;
			}
		}
		return false;
	}

	public Chunk createChunk(Matcher matcher) {
		return new ParagraphChunk(matcher.group(1), isInsideItem);
	}

}
